@@ -1,8 +1,4 @@
-// Copyright Timothy H. Keitt 2018
-// Please see the LICENSE file in the R package
-// By using this header, you agree to cite this
-// work in your product using the DOI found at
-// https://doi.org/10.5281/zenodo.1213797
+// Copyright Timothy H. Keitt 2020
 
 #ifndef __KDTOOLS_H__
 #define __KDTOOLS_H__
@@ -19,14 +15,18 @@
 #include <tuple>
 #include <cmath>
 
+// #define USE_CIRCULAR_LEXICOGRAPHIC_COMPARE
+// #define NO_TUPLEMAPR
+
+#ifndef NO_TUPLEMAPR
+#include "tuplemapr.h"
+#endif // NO_TUPLEMAPR
+
+namespace keittlab {
 namespace kdtools {
 
-template <typename T>
-struct ndim
-{
-  using U = typename std::remove_pointer<T>::type;
-  static constexpr auto value = std::tuple_size<U>::value;
-};
+template <typename T> static constexpr auto
+  ndim = std::tuple_size_v<typename std::remove_pointer_t<T>>;
 
 // Specialize for non-numeric types
 // TODO: User defined distance functions
@@ -48,6 +48,7 @@ namespace detail {
 using std::abs;
 using std::get;
 using std::next;
+using std::prev;
 using std::pair;
 using std::size_t;
 using std::thread;
@@ -56,9 +57,9 @@ using std::is_same;
 using std::distance;
 using std::enable_if;
 using std::partition;
-using std::is_pointer;
 using std::nth_element;
-using std::tuple_element;
+using std::is_pointer_v;
+using std::tuple_size_v;
 using std::numeric_limits;
 using std::priority_queue;
 using std::is_partitioned;
@@ -67,12 +68,6 @@ using std::iterator_traits;
 using std::partition_point;
 using std::placeholders::_1;
 
-template <typename T>
-struct is_not_pointer
-{
-  static constexpr bool value = !is_pointer<T>::value;
-};
-
 template <typename T>
 T middle_of(const T first, const T last)
 {
@@ -83,16 +78,41 @@ template <size_t I>
 struct less_nth
 {
   template <typename T>
-  typename enable_if<is_not_pointer<T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
+  bool operator()(const T& lhs, const T& rhs)
   {
-    return get<I>(lhs) < get<I>(rhs);
+    if constexpr (is_pointer_v<T>)
+      return get<I>(*lhs) < get<I>(*rhs);
+    else
+      return get<I>(lhs) < get<I>(rhs);
   }
-  template <typename T>
-  typename enable_if<is_pointer<T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
+  template <typename T, typename U>
+  bool operator()(const pair<T, U>& lhs, const pair<T, U>& rhs)
   {
-    return get<I>(*lhs) < get<I>(*rhs);
+    if constexpr (is_pointer_v<T>)
+      return get<I>(*lhs.first) < get<I>(*rhs.first);
+    else
+      return get<I>(lhs.first) < get<I>(rhs.first);
+  }
+};
+
+template <size_t I>
+struct less_radius_nth
+{
+  template <typename T, typename U>
+  bool operator()(const T& lhs, const T& rhs, const U radius)
+  {
+    if constexpr (is_pointer_v<T>)
+      return scalar_diff(get<I>(*lhs), get<I>(*rhs)) < radius;
+    else
+      return scalar_diff(get<I>(lhs), get<I>(rhs)) < radius;
+  }
+  template <typename T, typename U, typename V>
+  bool operator()(const pair<T, U>& lhs, const pair<T, U>& rhs, const V radius)
+  {
+    if constexpr (is_pointer_v<T>)
+      return scalar_diff(get<I>(*lhs.first), get<I>(*rhs.first)) < radius;
+    else
+      return scalar_diff(get<I>(lhs.first), get<I>(rhs.first)) < radius;
   }
 };
 
@@ -100,16 +120,20 @@ template <size_t I>
 struct equal_nth
 {
   template <typename T>
-  typename enable_if<is_not_pointer<T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
+  bool operator()(const T& lhs, const T& rhs)
   {
-    return get<I>(lhs) == get<I>(rhs);
+    if constexpr (is_pointer_v<T>)
+      return get<I>(*lhs) == get<I>(*rhs);
+    else
+      return get<I>(lhs) == get<I>(rhs);
   }
-  template <typename T>
-  typename enable_if<is_pointer<T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
+  template <typename T, typename U>
+  bool operator()(const pair<T, U>& lhs, const pair<T, U>& rhs)
   {
-    return get<I>(*lhs) == get<I>(*rhs);
+    if constexpr (is_pointer_v<T>)
+      return get<I>(*lhs.first) == get<I>(*rhs.first);
+    else
+      return get<I>(lhs.first) == get<I>(rhs.first);
   }
 };
 
@@ -119,16 +143,20 @@ struct pred_nth
   Pred m_pred;
   pred_nth(const Pred& pred) : m_pred(pred) {}
   template <typename T>
-  typename enable_if<is_not_pointer<T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
+  bool operator()(const T& lhs, const T& rhs)
   {
-    return m_pred(get<I>(lhs), get<I>(rhs));
+    if constexpr (is_pointer_v<T>)
+      return m_pred(get<I>(*lhs), get<I>(*rhs));
+    else
+      return m_pred(get<I>(lhs), get<I>(rhs));
   }
-  template <typename T>
-  typename enable_if<is_pointer<T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
+  template <typename T, typename U>
+  bool operator()(const pair<T, U>& lhs, const pair<T, U>& rhs)
   {
-    return m_pred(get<I>(*lhs), get<I>(*rhs));
+    if constexpr (is_pointer_v<T>)
+      return m_pred(get<I>(*lhs.first), get<I>(*rhs.first));
+    else
+      return m_pred(get<I>(lhs.first), get<I>(rhs.first));
   }
 };
 
@@ -139,73 +167,92 @@ pred_nth<Pred, I> make_pred_nth(const Pred& pred)
 }
 
 template <size_t I, typename T>
-typename enable_if<is_not_pointer<T>::value, double>::type
-dist_nth(const T& lhs, const T& rhs)
+double dist_nth(const T& lhs, const T& rhs)
 {
-  return scalar_dist(get<I>(lhs), get<I>(rhs));
+  if constexpr (is_pointer_v<T>)
+    return scalar_dist(get<I>(*lhs), get<I>(*rhs));
+  else
+    return scalar_dist(get<I>(lhs), get<I>(rhs));
+}
+
+template <size_t I, typename T, typename U>
+double dist_nth(const pair<T, U>& lhs, const pair<T, U>& rhs)
+{
+  if constexpr (is_pointer_v<T>)
+    return scalar_dist(get<I>(*lhs.first), get<I>(*rhs.first));
+  else
+    return scalar_dist(get<I>(lhs.first), get<I>(rhs.first));
 }
 
 template <size_t I, typename T>
-typename enable_if<is_pointer<T>::value, double>::type
-dist_nth(const T& lhs, const T& rhs)
+double diff_nth(const T& lhs, const T& rhs)
 {
-  return scalar_dist(get<I>(*lhs), get<I>(*rhs));
+  if constexpr (is_pointer_v<T>)
+    return scalar_diff(get<I>(*lhs), get<I>(*rhs));
+  else
+    return scalar_diff(get<I>(lhs), get<I>(rhs));
+}
+
+template <size_t I, typename T, typename U>
+double diff_nth(const pair<T, U>& lhs, const pair<T, U>& rhs)
+{
+  if constexpr (is_pointer_v<T>)
+    return scalar_diff(get<I>(*lhs.first), get<I>(*rhs.first));
+  else
+    return scalar_diff(get<I>(lhs.first), get<I>(rhs.first));
 }
 
 template <size_t I, typename T>
-typename enable_if<is_not_pointer<T>::value, double>::type
-diff_nth(const T& lhs, const T& rhs)
-{
-  return scalar_diff(get<I>(lhs), get<I>(rhs));
-}
+static constexpr auto next_dim = (I + 1) % ndim<T>;
 
-template <size_t I, typename T>
-typename enable_if<is_pointer<T>::value, double>::type
-diff_nth(const T& lhs, const T& rhs)
-{
-  return scalar_diff(get<I>(*lhs), get<I>(*rhs));
-}
+template<size_t I, typename T>
+static constexpr auto is_last = (I == (ndim<T> - 1));
 
-template <size_t I, size_t N>
-struct incr_wrap
-{
-  static constexpr auto value = (I + 1) % N;
-};
-
-template <size_t I, typename T>
-struct next_dim
-{
-  static constexpr auto value = incr_wrap<I, ndim<T>::value>::value;
-};
-
-template <size_t I, typename TupleType>
-struct is_not_last
-{
-  static constexpr auto value = (I != (ndim<TupleType>::value - 1));
-};
-
-template<size_t I, typename TupleType>
-struct is_last
-{
-  static constexpr auto value = (I == (ndim<TupleType>::value - 1));
-};
+#ifdef USE_CIRCULAR_LEXICOGRAPHIC_COMPARE
 
 template <size_t I, size_t K = 0>
 struct kd_less
 {
   template <typename T>
-  typename enable_if<is_not_last<K, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs) const
+  bool operator()(const T& lhs, const T& rhs) const
   {
-    constexpr auto J = next_dim<I, T>::value;
-    return equal_nth<I>()(lhs, rhs) ?
-      kd_less<J, K + 1>()(lhs, rhs) :
-        less_nth<I>()(lhs, rhs);
+    if constexpr (is_last<K, T>) {
+      return less_nth<I>()(lhs, rhs);
+    } else {
+      constexpr auto J = next_dim<I, T>;
+      return equal_nth<I>()(lhs, rhs) ?
+        kd_less<J, K + 1>()(lhs, rhs) :
+          less_nth<I>()(lhs, rhs);
+    }
   }
+};
+
+template <typename Pred, size_t I, size_t K = 0>
+struct kd_compare
+{
+  Pred m_pred;
+  kd_compare(const Pred& pred) : m_pred(pred) {}
   template <typename T>
-  typename enable_if<is_last<K, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs) const
+  bool operator()(const T& lhs, const T& rhs) const
   {
+    if constexpr (is_last<K, T>) {
+      return make_pred_nth<I>(m_pred)(lhs, rhs);
+    } else {
+      constexpr auto J = next_dim<I, T>;
+      auto pred = make_pred_nth<I>(m_pred);
+      return !pred(lhs, rhs) && !pred(rhs, lhs) ?
+        kd_compare<Pred, J, K + 1>(m_pred)(lhs, rhs) : pred(lhs, rhs);
+    }
+  }
+};
+
+#else // USE_CIRCULAR_LEXICOGRAPHIC_COMPARE
+
+template <size_t I>
+struct kd_less
+{
+  template <typename T>
+  bool operator()(const T& lhs, const T& rhs) const {
     return less_nth<I>()(lhs, rhs);
   }
 };
@@ -216,22 +263,14 @@ struct kd_compare
   Pred m_pred;
   kd_compare(const Pred& pred) : m_pred(pred) {}
   template <typename T>
-  typename enable_if<is_not_last<K, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs) const
+  bool operator()(const T& lhs, const T& rhs) const
   {
-    auto pred = make_pred_nth<I>(m_pred);
-    constexpr auto J = next_dim<I, T>::value;
-    return !pred(lhs, rhs) && !pred(lhs, rhs) ?
-      kd_compare<Pred, J, K + 1>(m_pred)(lhs, rhs) : pred(lhs, rhs);
-  }
-  template <typename T>
-  typename enable_if<is_last<K, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs) const
-  {
-    return make_pred_nth<I>(m_pred)(lhs, rhs);
+      return make_pred_nth<I>(m_pred)(lhs, rhs);
   }
 };
 
+#endif // USE_CIRCULAR_LEXICOGRAPHIC_COMPARE
+
 template <size_t I, typename Pred>
 kd_compare<Pred, I> make_kd_compare(const Pred& pred)
 {
@@ -241,105 +280,53 @@ kd_compare<Pred, I> make_kd_compare(const Pred& pred)
 template <typename T>
 using iter_value_t = typename iterator_traits<T>::value_type;
 
-template <size_t I, typename Iter>
-Iter find_pivot(Iter first, Iter last)
-{
-  using T = iter_value_t<Iter>;
-  auto pivot = middle_of(first, last);
-  return partition_point(first, pivot, [&](const T& x){
-    return less_nth<I>()(x, *pivot);
-  });
-}
-
 template <typename Iter, typename Pred>
-Iter adjust_pivot(Iter first, Iter pivot, Pred pred)
+Iter median_part(Iter first, Iter last, Pred pred)
 {
-  using T = iter_value_t<Iter>;
-  return partition(first, pivot, [&](const T& x){
-    return pred(x, *pivot);
-  });
+  auto pivot = middle_of(first, last);
+  nth_element(first, pivot, last, pred);
+  return pivot;
 }
 
 template <size_t I, typename Iter>
 void kd_sort(Iter first, Iter last)
 {
   using TupleType = iter_value_t<Iter>;
-  constexpr auto J = next_dim<I, TupleType>::value;
+  constexpr auto J = next_dim<I, TupleType>;
   if (distance(first, last) > 1)
   {
     auto pred = kd_less<I>();
-    auto pivot = middle_of(first, last);
-    nth_element(first, pivot, last, pred);
-    pivot = adjust_pivot(first, pivot, pred);
+    auto pivot = median_part(first, last, pred);
     kd_sort<J>(next(pivot), last);
     kd_sort<J>(first, pivot);
   }
 }
 
-template <typename Iter, typename Pred>
-bool check_partition(Iter first, Iter pivot, Iter last, Pred pred)
-{
-  using T = iter_value_t<Iter>;
-  return is_partitioned(first, last, [&](const T& x){
-    return pred(x, *pivot);
-  });
-}
-
-template <size_t I, typename Iter>
-bool kd_is_sorted(Iter first, Iter last)
-{
-  using TupleType = iter_value_t<Iter>;
-  constexpr auto J = next_dim<I, TupleType>::value;
-  if (distance(first, last) < 2) return true;
-  auto pred = kd_less<I>();
-  auto pivot = find_pivot<I>(first, last);
-  return check_partition(first, pivot, last, pred) &&
-    kd_is_sorted<J>(first, pivot) &&
-    kd_is_sorted<J>(next(pivot), last);
-}
-
 template <size_t I, typename Iter, typename Compare>
 void kd_sort(Iter first, Iter last, const Compare& comp)
 {
   using TupleType = iter_value_t<Iter>;
-  constexpr auto J = next_dim<I, TupleType>::value;
+  constexpr auto J = next_dim<I, TupleType>;
   if (distance(first, last) > 1)
   {
-    auto pivot = middle_of(first, last);
     auto pred = make_kd_compare<I>(comp);
-    nth_element(first,  pivot,  last,  pred);
-    pivot = adjust_pivot(first, pivot, pred);
+    auto pivot = median_part(first, last, pred);
     kd_sort<J>(next(pivot), last, comp);
     kd_sort<J>(first, pivot, comp);
   }
 }
 
-template <size_t I, typename Iter, typename Compare>
-bool kd_is_sorted(Iter first, Iter last, const Compare& comp)
-{
-  using TupleType = iter_value_t<Iter>;
-  constexpr auto J = next_dim<I, TupleType>::value;
-  if (distance(first, last) < 2) return true;
-  auto pred = make_kd_compare<I>(comp);
-  auto pivot = find_pivot<I>(first, last);
-  return check_partition(first, pivot, last, pred) &&
-    kd_is_sorted<J>(first, pivot, comp) &&
-    kd_is_sorted<J>(next(pivot), last, comp);
-}
-
 template <size_t I, typename Iter>
 void kd_sort_threaded(Iter first, Iter last,
                       int max_threads = std::thread::hardware_concurrency(),
                       int thread_depth = 1)
 {
   using TupleType = iter_value_t<Iter>;
-  constexpr auto J = next_dim<I, TupleType>::value;
+  constexpr auto J = next_dim<I, TupleType>;
   if (distance(first, last) > 1)
   {
     auto pred = kd_less<I>();
-    auto pivot = middle_of(first, last);
-    nth_element(first, pivot, last, pred);
-    pivot = adjust_pivot(first, pivot, pred);
+    auto pivot = median_part(first, last, pred);
     if ((1 << thread_depth) <= max_threads)
     {
       thread t(kd_sort_threaded<J, Iter>,
@@ -361,16 +348,14 @@ void kd_sort_threaded(Iter first, Iter last, const Compare& comp,
                       int thread_depth = 1)
 {
   using TupleType = iter_value_t<Iter>;
-  constexpr auto J = next_dim<I, TupleType>::value;
+  constexpr auto J = next_dim<I, TupleType>;
   if (distance(first, last) > 1)
   {
-    auto pivot = middle_of(first, last);
     auto pred = make_kd_compare<I>(comp);
-    nth_element(first, pivot, last, pred);
-    pivot = adjust_pivot(first, pivot, pred);
+    auto pivot = median_part(first, last, pred);
     if ((1 << thread_depth) <= max_threads)
     {
-      thread t(kd_sort_threaded<J, Iter>,
+      thread t(kd_sort_threaded<J, Iter, Compare>,
                next(pivot), last, comp, max_threads, thread_depth + 1);
       kd_sort_threaded<J>(first, pivot, comp, max_threads, thread_depth + 1);
       t.join();
@@ -383,20 +368,113 @@ void kd_sort_threaded(Iter first, Iter last, const Compare& comp,
   }
 }
 
+template <typename Iter, typename Pred>
+bool check_partition(Iter first, Iter pivot, Iter last, Pred pred)
+{
+  while (first != pivot) if (pred(*pivot, *first++)) return false;
+  while (first != last) if (pred(*first++, *pivot)) return false;
+  return true;
+}
+
+template <size_t I, typename Iter>
+bool kd_is_sorted(Iter first, Iter last)
+{
+  using TupleType = iter_value_t<Iter>;
+  constexpr auto J = next_dim<I, TupleType>;
+  if (distance(first, last) < 2) return true;
+  auto pred = kd_less<I>();
+  auto pivot = middle_of(first, last);
+  return check_partition(first, pivot, last, pred) &&
+    kd_is_sorted<J>(first, pivot) &&
+    kd_is_sorted<J>(next(pivot), last);
+}
+
+template <size_t I, typename Iter, typename Compare>
+bool kd_is_sorted(Iter first, Iter last, const Compare& comp)
+{
+  using TupleType = iter_value_t<Iter>;
+  constexpr auto J = next_dim<I, TupleType>;
+  if (distance(first, last) < 2) return true;
+  auto pred = make_kd_compare<I>(comp);
+  auto pivot = middle_of(first, last);
+  return check_partition(first, pivot, last, pred) &&
+    kd_is_sorted<J>(first, pivot, comp) &&
+    kd_is_sorted<J>(next(pivot), last, comp);
+}
+
+template <size_t I, typename Iter>
+bool kd_is_sorted_threaded(Iter first, Iter last,
+                           int max_threads = std::thread::hardware_concurrency(),
+                           int thread_depth = 1)
+{
+  using TupleType = iter_value_t<Iter>;
+  constexpr auto J = next_dim<I, TupleType>;
+  if (distance(first, last) < 2) return true;
+  auto pred = kd_less<I>();
+  auto pivot = middle_of(first, last);
+  if (check_partition(first, pivot, last, pred)) {
+    if ((1 << thread_depth) <= max_threads)
+    {
+      bool res_left, res_right;
+      thread t([=, &res_left](){
+        res_left = kd_is_sorted_threaded<J>(first, pivot, max_threads, thread_depth + 1);
+      });
+      res_right = kd_is_sorted_threaded<J>(next(pivot), last, max_threads, thread_depth + 1);
+      t.join();
+      return res_left && res_right;
+    }
+    else
+    {
+      return kd_is_sorted<J>(first, pivot) && kd_is_sorted<J>(next(pivot), last);
+    }
+  } else {
+    return false;
+  }
+}
+
+template <size_t I, typename Iter, typename Compare>
+bool kd_is_sorted_threaded(Iter first, Iter last, const Compare& comp,
+                           int max_threads = std::thread::hardware_concurrency(),
+                           int thread_depth = 1)
+{
+  using TupleType = iter_value_t<Iter>;
+  constexpr auto J = next_dim<I, TupleType>;
+  if (distance(first, last) < 2) return true;
+  auto pred = make_kd_compare<I>(comp);
+  auto pivot = middle_of(first, last);
+  if (check_partition(first, pivot, last, pred)) {
+    if ((1 << thread_depth) <= max_threads)
+    {
+      bool res_left, res_right;
+      thread t([=, &res_left](){
+        res_left = kd_is_sorted_threaded<J>(first, pivot, comp, max_threads, thread_depth + 1);
+      });
+      res_right = kd_is_sorted_threaded<J>(next(pivot), last, comp, max_threads, thread_depth + 1);
+      t.join();
+      return res_left && res_right;
+    }
+    else
+    {
+      return kd_is_sorted<J>(first, pivot, comp) && kd_is_sorted<J>(next(pivot), last, comp);
+    }
+  } else {
+    return false;
+  }
+}
+
+#ifdef NO_TUPLEMAPR
+
 template <size_t I>
 struct all_less_
 {
   template <typename T>
-  typename enable_if<is_not_last<I, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs) const
+  bool operator()(const T& lhs, const T& rhs) const
   {
-    return less_nth<I>()(lhs, rhs) && all_less_<I + 1>()(lhs, rhs);
-  }
-  template <typename T>
-  typename enable_if<is_last<I, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs) const
-  {
-    return less_nth<I>()(lhs, rhs);
+    if constexpr (is_last<I, T>) {
+      return less_nth<I>()(lhs, rhs);
+    } else {
+      return less_nth<I>()(lhs, rhs) && all_less_<I + 1>()(lhs, rhs);
+    }
   }
 };
 
@@ -410,16 +488,13 @@ template <size_t I>
 struct none_less_
 {
   template <typename T>
-  typename enable_if<is_not_last<I, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
+  bool operator()(const T& lhs, const T& rhs)
   {
-    return !less_nth<I>()(lhs, rhs) && none_less_<I + 1>()(lhs, rhs);
-  }
-  template <typename T>
-  typename enable_if<is_last<I, T>::value, bool>::type
-  operator()(const T& lhs, const T& rhs)
-  {
-    return !less_nth<I>()(lhs, rhs);
+    if constexpr (is_last<I, T>) {
+      return !less_nth<I>()(lhs, rhs);
+    } else {
+      return !less_nth<I>()(lhs, rhs) && none_less_<I + 1>()(lhs, rhs);
+    }
   }
 };
 
@@ -429,61 +504,18 @@ bool none_less(const TupleType& lhs, const TupleType& rhs)
   return none_less_<0>()(lhs, rhs);
 }
 
-template <size_t I, typename Iter, typename TupleType>
-Iter kd_lower_bound(Iter first, Iter last, const TupleType& value)
-{
-  constexpr auto J = next_dim<I, TupleType>::value;
-  if (distance(first, last) > 1)
-  {
-    auto pivot = find_pivot<I>(first, last);
-    if (none_less(*pivot, value))
-      return kd_lower_bound<J>(first, pivot, value);
-    if (all_less(*pivot, value))
-      return kd_lower_bound<J>(next(pivot), last, value);
-    auto it = kd_lower_bound<J>(first, pivot, value);
-    if (it != last && none_less(*it, value)) return it;
-    it = kd_lower_bound<J>(next(pivot), last, value);
-    if (it != last && none_less(*it, value)) return it;
-    return last;
-  }
-  return first != last && none_less(*first, value) ? first : last;
-}
-
-template <size_t I, typename Iter, typename TupleType>
-Iter kd_upper_bound(Iter first, Iter last, const TupleType& value)
-{
-  constexpr auto J = next_dim<I, TupleType>::value;
-  if (distance(first, last) > 1)
-  {
-    auto pivot = find_pivot<I>(first, last);
-    if (all_less(value, *pivot))
-      return kd_upper_bound<J>(first, pivot, value);
-    if (none_less(value, *pivot))
-      return kd_upper_bound<J>(next(pivot), last, value);
-    auto it = kd_upper_bound<J>(first, pivot, value);
-    if (it != last && all_less(value, *it)) return it;
-    it = kd_upper_bound<J>(next(pivot), last, value);
-    if (it != last && all_less(value, *it)) return it;
-    return last;
-  }
-  return first != last && all_less(value, *first) ? first : last;
-}
-
 template <size_t I>
 struct sum_of_squares_
 {
   template <typename TupleType>
-  typename enable_if<is_not_last<I, TupleType>::value, double>::type
-  operator()(const TupleType& lhs, const TupleType& rhs) const
+  double operator()(const TupleType& lhs, const TupleType& rhs) const
   {
-    using next_ = sum_of_squares_<I + 1>;
-    return std::pow(diff_nth<I>(rhs, lhs), 2) + next_()(lhs, rhs);
-  }
-  template <typename TupleType>
-  typename enable_if<is_last<I, TupleType>::value, double>::type
-  operator()(const TupleType& lhs, const TupleType& rhs) const
-  {
-    return std::pow(diff_nth<I>(rhs, lhs), 2);
+    if constexpr (is_last<I, TupleType>) {
+      return std::pow(diff_nth<I>(rhs, lhs), 2);
+    } else {
+      using next_ = sum_of_squares_<I + 1>;
+      return std::pow(diff_nth<I>(rhs, lhs), 2) + next_()(lhs, rhs);
+    }
   }
 };
 
@@ -499,43 +531,111 @@ double l2dist(const TupleType& lhs, const TupleType& rhs)
   return std::sqrt(sum_of_squares(lhs, rhs));
 }
 
-template <size_t I, typename Iter, typename TupleType>
-Iter kd_nearest_neighbor(Iter first, Iter last, const TupleType& value)
+#else // NO_TUPLEMAPR
+
+using tuple::all_less;
+using tuple::none_less;
+
+template <typename TupleType>
+double sum_of_squares(const TupleType& lhs, const TupleType& rhs)
 {
-  constexpr auto J = next_dim<I, TupleType>::value;
+  return tuple::sum_sq_diff(lhs, rhs);
+}
+
+template <typename TupleType>
+double l2dist(const TupleType& lhs, const TupleType& rhs)
+{
+  return tuple::euclidean_distance(lhs, rhs);
+}
+
+#endif // NO_TUPLEMAPR
+
+template <size_t I, typename Iter, typename TupleType>
+Iter kd_lower_bound(Iter first, Iter last, const TupleType& key)
+{
+  constexpr auto J = next_dim<I, TupleType>;
   if (distance(first, last) > 1)
   {
-    auto pivot = find_pivot<I>(first, last);
-    auto search_left = less_nth<I>()(value, *pivot);
-    auto search = search_left ?
-      kd_nearest_neighbor<J>(first, pivot, value) :
-        kd_nearest_neighbor<J>(next(pivot), last, value);
-    auto min_dist = l2dist(*pivot, value);
-    if (search == last) search = pivot;
-    else
-    {
-      auto sdist = l2dist(*search, value);
-      if (sdist < min_dist) min_dist = sdist;
-      else search = pivot;
-    }
-    if (dist_nth<I>(value, *pivot) < min_dist)
-    {
+    auto pivot = middle_of(first, last);
+    if (none_less(*pivot, key))
+      return kd_lower_bound<J>(first, pivot, key);
+    if (all_less(*pivot, key))
+      return kd_lower_bound<J>(next(pivot), last, key);
+    auto it = kd_lower_bound<J>(first, pivot, key);
+    if (it != last && none_less(*it, key)) return it;
+    it = kd_lower_bound<J>(next(pivot), last, key);
+    if (it != last && none_less(*it, key)) return it;
+    return last;
+  }
+  return first != last && none_less(*first, key) ? first : last;
+}
+
+template <size_t I, typename Iter, typename TupleType>
+Iter kd_upper_bound(Iter first, Iter last, const TupleType& key)
+{
+  constexpr auto J = next_dim<I, TupleType>;
+  if (distance(first, last) > 1)
+  {
+    auto pivot = middle_of(first, last);
+    if (all_less(key, *pivot))
+      return kd_upper_bound<J>(first, pivot, key);
+    if (none_less(key, *pivot))
+      return kd_upper_bound<J>(next(pivot), last, key);
+    auto it = kd_upper_bound<J>(first, pivot, key);
+    if (it != last && all_less(key, *it)) return it;
+    it = kd_upper_bound<J>(next(pivot), last, key);
+    if (it != last && all_less(key, *it)) return it;
+    return last;
+  }
+  return first != last && all_less(key, *first) ? first : last;
+}
+
+template <size_t I, typename Iter, typename TupleType>
+Iter kd_nearest_neighbor(Iter first, Iter last, const TupleType& key)
+{
+  constexpr auto J = next_dim<I, TupleType>;
+  if (distance(first, last) > 1)
+  {
+    auto pivot = middle_of(first, last);
+    if (equal_nth<I>()(*pivot, key)) {
+      auto left_res = kd_nearest_neighbor<J>(first, pivot, key);
+      auto right_res = kd_nearest_neighbor<J>(next(pivot), last, key);
+      if (l2dist(*right_res, key) < l2dist(*left_res, key)) {
+        return right_res;
+      } else {
+        return left_res;
+      }
+    } else {
+      auto search_left = less_nth<I>()(key, *pivot);
+      auto search = search_left ?
+        kd_nearest_neighbor<J>(first, pivot, key) :
+          kd_nearest_neighbor<J>(next(pivot), last, key);
+      auto min_dist = l2dist(*pivot, key);
+      if (search == last) {
+        search = pivot;
+      } else {
+        auto sdist = l2dist(*search, key);
+        if (sdist < min_dist) min_dist = sdist;
+        else search = pivot;
+      }
+      if (dist_nth<I>(key, *pivot) < min_dist) {
       auto s2 = search_left ?
-        kd_nearest_neighbor<J>(next(pivot), last, value) :
-          kd_nearest_neighbor<J>(first, pivot, value);
-      if (s2 != last && l2dist(*s2, value) < min_dist) search = s2;
+        kd_nearest_neighbor<J>(next(pivot), last, key) :
+          kd_nearest_neighbor<J>(first, pivot, key);
+      if (s2 != last && l2dist(*s2, key) < min_dist) search = s2;
+      }
+     return search;
     }
-    return search;
   }
   return first;
 }
 
 template <typename TupleType>
-bool within(const TupleType& value,
+bool within(const TupleType& key,
             const TupleType& lower,
             const TupleType& upper)
 {
-  return none_less(value, lower) && all_less(value, upper);
+  return none_less(key, lower) && all_less(key, upper);
 }
 
 template <size_t I,
@@ -549,8 +649,8 @@ void kd_range_query(Iter first, Iter last,
 {
   if (distance(first, last) > 32) {
     auto pred = less_nth<I>();
-    auto pivot = find_pivot<I>(first, last);
-    constexpr auto J = next_dim<I, TupleType>::value;
+    auto pivot = middle_of(first, last);
+    constexpr auto J = next_dim<I, TupleType>;
     if (within(*pivot, lower, upper)) *outp++ = *pivot;
     if (!pred(*pivot, lower)) // search left
       kd_range_query<J>(first, pivot, lower, upper, outp);
@@ -575,8 +675,8 @@ void kd_rq_iters(Iter first, Iter last,
 {
   if (distance(first, last) > 32) {
     auto pred = less_nth<I>();
-    auto pivot = find_pivot<I>(first, last);
-    constexpr auto J = next_dim<I, TupleType>::value;
+    auto pivot = middle_of(first, last);
+    constexpr auto J = next_dim<I, TupleType>;
     if (within(*pivot, lower, upper)) *outp++ = pivot;
     if (!pred(*pivot, lower)) // search left
       kd_rq_iters<J>(first, pivot, lower, upper, outp);
@@ -591,12 +691,72 @@ void kd_rq_iters(Iter first, Iter last,
   return;
 }
 
+template <size_t I,
+          typename Iter,
+          typename TupleType,
+          typename OutIter>
+void kd_range_query(Iter first, Iter last,
+                    const TupleType& center,
+                    const double radius,
+                    OutIter outp)
+{
+  if (distance(first, last) > 32) {
+    auto pred = less_radius_nth<I>();
+    auto pivot = middle_of(first, last);
+    constexpr auto J = next_dim<I, TupleType>;
+    if (l2dist(*pivot, center) <= radius) *outp++ = *pivot;
+    if (!pred(*pivot, center, -radius)) // search left
+      kd_range_query<J>(first, pivot, center, radius, outp);
+    if (!pred(center, *pivot, -radius)) // search right
+      kd_range_query<J>(next(pivot), last, center, radius, outp);
+  } else {
+    copy_if(first, last, outp, [&](const TupleType& x){
+      return l2dist(x, center) <= radius;
+    });
+  }
+  return;
+}
+
+template <size_t I,
+          typename Iter,
+          typename TupleType,
+          typename OutIter>
+void kd_rq_iters(Iter first, Iter last,
+                 const TupleType& center,
+                 const double radius,
+                 OutIter outp)
+{
+  if (distance(first, last) > 32) {
+    auto pred = less_radius_nth<I>();
+    auto pivot = middle_of(first, last);
+    constexpr auto J = next_dim<I, TupleType>;
+    if (l2dist(*pivot, center) <= radius) *outp++ = pivot;
+    if (!pred(*pivot, center, -radius)) // search left
+      kd_rq_iters<J>(first, pivot, center, radius, outp);
+    if (pred(*pivot, center, radius)) // search right
+      kd_rq_iters<J>(next(pivot), last, center, radius, outp);
+  } else {
+    while (first != last) {
+      if (l2dist(*first, center) <= radius) *outp++ = first;
+      ++first;
+    }
+  }
+  return;
+}
+
+template <typename Key, typename Iter>
+struct less_key {
+  bool operator()(const pair<Key, Iter>& lhs, const pair<Key, Iter>& rhs) {
+    return lhs.first < rhs.first;
+  }
+};
+
 template <typename Iter, typename Key = double>
 struct n_best
 {
   using qmem_t = pair<Key, Iter>;
   using qcont_t = vector<qmem_t>;
-  using qcomp_t = less_nth<0>;
+  using qcomp_t = less_key<Key, Iter>;
   using queue_t = priority_queue<qmem_t, qcont_t, qcomp_t>;
   size_t m_n;
   queue_t m_q;
@@ -609,8 +769,12 @@ struct n_best
   }
   void add(Key dist, Iter it)
   {
-    m_q.emplace(dist, it);
-    if (m_q.size() > m_n) m_q.pop();
+    if (m_q.size() < m_n) {
+      m_q.emplace(dist, it);
+    } else if (dist < m_q.top().first) {
+      m_q.emplace(dist, it);
+      m_q.pop();
+    }
   }
   template <typename OutIter>
   void copy_to(OutIter outp)
@@ -630,6 +794,15 @@ struct n_best
       m_q.pop();
     }
   }
+  template <typename OutIter>
+  void copy_dist_to(OutIter outp)
+  {
+    while (!m_q.empty())
+    {
+      *outp++ = m_q.top();
+      m_q.pop();
+    }
+  }
 };
 
 template <size_t I,
@@ -637,26 +810,31 @@ template <size_t I,
           typename TupleType,
           typename QType>
 void knn(Iter first, Iter last,
-         const TupleType& value,
+         const TupleType& key,
          QType& Q)
 {
   switch(distance(first, last)) {
-  case 1 : Q.add(l2dist(*first, value), first);
+  case 1 : Q.add(l2dist(*first, key), first);
   case 0 : return; } // switch end
-  auto pivot = find_pivot<I>(first, last);
-  Q.add(l2dist(*pivot, value), pivot);
-  auto search_left = less_nth<I>()(value, *pivot);
-  constexpr auto J = next_dim<I, TupleType>::value;
-  if (search_left)
-    knn<J>(first, pivot, value, Q);
-  else
-    knn<J>(next(pivot), last, value, Q);
-  if (dist_nth<I>(value, *pivot) <= Q.max_key())
-  {
+  auto pivot = middle_of(first, last);
+  Q.add(l2dist(*pivot, key), pivot);
+  constexpr auto J = next_dim<I, TupleType>;
+  if (equal_nth<I>()(*pivot, key)) {
+    knn<J>(first, pivot, key, Q);
+    knn<J>(next(pivot), last, key, Q);
+  } else {
+    auto search_left = less_nth<I>()(key, *pivot);
     if (search_left)
-      knn<J>(next(pivot), last, value, Q);
+      knn<J>(first, pivot, key, Q);
     else
-      knn<J>(first, pivot, value, Q);
+      knn<J>(next(pivot), last, key, Q);
+    if (dist_nth<I>(key, *pivot) <= Q.max_key())
+    {
+      if (search_left)
+        knn<J>(next(pivot), last, key, Q);
+      else
+        knn<J>(first, pivot, key, Q);
+    }
   }
 }
 
@@ -669,12 +847,11 @@ using detail::none_less;
 using detail::within;
 
 using detail::middle_of;
-using detail::find_pivot;
+using detail::median_part;
 
 using detail::is_last;
-using detail::is_not_last;
-using detail::incr_wrap;
 using detail::next_dim;
+using detail::iter_value_t;
 
 using detail::kd_less;
 using detail::less_nth;
@@ -687,6 +864,8 @@ using detail::make_kd_compare;
 using detail::l2dist;
 using detail::sum_of_squares;
 
+using detail::n_best;
+
 } // namespace utils
 
 template <typename Iter>
@@ -707,18 +886,30 @@ void kd_sort(Iter first, Iter last)
   detail::kd_sort<0>(first, last);
 }
 
-template <typename Iter>
-bool kd_is_sorted(Iter first, Iter last)
-{
-  return detail::kd_is_sorted<0>(first, last);
-}
-
 template <typename Iter, typename Compare>
 void kd_sort(Iter first, Iter last, const Compare& comp)
 {
   detail::kd_sort<0>(first, last, comp);
 }
 
+template <typename Iter>
+void kd_sort_threaded(Iter first, Iter last)
+{
+  detail::kd_sort_threaded<0>(first, last);
+}
+
+template <typename Iter, typename Compare>
+void kd_sort_threaded(Iter first, Iter last, const Compare& comp)
+{
+  detail::kd_sort_threaded<0>(first, last, comp);
+}
+
+template <typename Iter>
+bool kd_is_sorted(Iter first, Iter last)
+{
+  return detail::kd_is_sorted<0>(first, last);
+}
+
 template <typename Iter, typename Compare>
 bool kd_is_sorted(Iter first, Iter last, const Compare& comp)
 {
@@ -726,41 +917,47 @@ bool kd_is_sorted(Iter first, Iter last, const Compare& comp)
 }
 
 template <typename Iter>
-void kd_sort_threaded(Iter first, Iter last)
+bool kd_is_sorted_threaded(Iter first, Iter last)
 {
-  detail::kd_sort_threaded<0>(first, last);
+  return detail::kd_is_sorted_threaded<0>(first, last);
 }
 
-template <typename Iter, typename Value>
-Iter kd_lower_bound(Iter first, Iter last, const Value& value)
+template <typename Iter, typename Compare>
+bool kd_is_sorted_threaded(Iter first, Iter last, const Compare& comp)
 {
-  return detail::kd_lower_bound<0>(first, last, value);
+  return detail::kd_is_sorted_threaded<0>(first, last, comp);
 }
 
-template <typename Iter, typename Value>
-Iter kd_upper_bound(Iter first, Iter last, const Value& value)
+template <typename Iter, typename Key>
+Iter kd_lower_bound(Iter first, Iter last, const Key& key)
 {
-  return detail::kd_upper_bound<0>(first, last, value);
+  return detail::kd_lower_bound<0>(first, last, key);
+}
+
+template <typename Iter, typename Key>
+Iter kd_upper_bound(Iter first, Iter last, const Key& key)
+{
+  return detail::kd_upper_bound<0>(first, last, key);
 }
 
 template <typename Iter, typename TupleType>
-bool kd_binary_search(Iter first, Iter last, const TupleType& value)
+bool kd_binary_search(Iter first, Iter last, const TupleType& key)
 {
-  first = detail::kd_lower_bound<0>(first, last, value);
-  return first != last && utils::none_less(value, *first);
+  first = detail::kd_lower_bound<0>(first, last, key);
+  return first != last && utils::none_less(key, *first);
 }
 
-template <typename Iter, typename Value>
-std::pair<Iter, Iter> kd_equal_range(Iter first, Iter last, const Value& value)
+template <typename Iter, typename Key>
+std::pair<Iter, Iter> kd_equal_range(Iter first, Iter last, const Key& key)
 {
-  return std::make_pair(detail::kd_lower_bound<0>(first, last, value),
-                        detail::kd_upper_bound<0>(first, last, value));
+  return std::make_pair(detail::kd_lower_bound<0>(first, last, key),
+                        detail::kd_upper_bound<0>(first, last, key));
 }
 
 template <typename Iter, typename TupleType>
-Iter kd_nearest_neighbor(Iter first, Iter last, const TupleType& value)
+Iter kd_nearest_neighbor(Iter first, Iter last, const TupleType& key)
 {
-  return detail::kd_nearest_neighbor<0>(first, last, value);
+  return detail::kd_nearest_neighbor<0>(first, last, key);
 }
 
 template <typename Iter,
@@ -788,27 +985,46 @@ void kd_rq_iters(Iter first, Iter last,
 template <typename Iter,
           typename TupleType,
           typename OutIter>
-void kd_nearest_neighbors(Iter first, Iter last,
-                          const TupleType& value,
-                          size_t n, OutIter outp)
+void kd_range_query(Iter first, Iter last,
+                    const TupleType& center,
+                    const double radius,
+                    OutIter outp)
+{
+  detail::kd_range_query<0>(first, last, center, radius, outp);
+}
+
+template <typename Iter,
+          typename TupleType,
+          typename OutIter>
+void kd_rq_iters(Iter first, Iter last,
+                 const TupleType& center,
+                 const double radius,
+                 OutIter outp)
+{
+  detail::kd_rq_iters<0>(first, last, center, radius, outp);
+}
+
+template <typename Iter,
+          typename TupleType,
+          typename OutIter>
+void kd_nearest_neighbors(Iter first, Iter last, const TupleType& key, size_t n, OutIter outp)
 {
   detail::n_best<Iter> Q(n);
-  detail::knn<0>(first, last, value, Q);
+  detail::knn<0>(first, last, key, Q);
   Q.copy_to(outp);
 }
 
 template <typename Iter,
           typename TupleType,
           typename OutIter>
-void kd_nn_iters(Iter first, Iter last,
-                 const TupleType& value,
-                 size_t n, OutIter outp)
+void kd_nn_iters(Iter first, Iter last, const TupleType& key, size_t n, OutIter outp)
 {
   detail::n_best<Iter> Q(n);
-  detail::knn<0>(first, last, value, Q);
+  detail::knn<0>(first, last, key, Q);
   Q.copy_iters_to(outp);
 }
 
 } // namespace kdtools
+} // namespace keittlab
 
 #endif // __KDTOOLS_H__
