@@ -1,6 +1,8 @@
 #include "arrayvec.h"
 #include "kdtools.h"
-using namespace kdtools;
+using namespace keittlab::kdtools;
+
+// [[Rcpp::plugins(cpp17)]]
 
 template <size_t I>
 List kd_sort__(List x, bool inplace, bool parallel)
@@ -19,7 +21,7 @@ List kd_sort__(List x, bool inplace, bool parallel)
 }
 
 // [[Rcpp::export]]
-List kd_sort_(List x, bool inplace = false, bool parallel = false)
+List kd_sort_(List x, bool inplace, bool parallel)
 {
   switch(arrayvec_dim(x)) {
   case 1: return kd_sort__<1>(x, inplace, parallel);
@@ -36,25 +38,28 @@ List kd_sort_(List x, bool inplace = false, bool parallel = false)
 }
 
 template <size_t I>
-bool kd_is_sorted__(List x)
+bool kd_is_sorted__(List x, bool parallel)
 {
   auto p = get_ptr<I>(x);
-  return kd_is_sorted(begin(*p), end(*p));
+  if (parallel)
+    return kd_is_sorted_threaded(begin(*p), end(*p));
+  else
+    return kd_is_sorted(begin(*p), end(*p));
 }
 
 // [[Rcpp::export]]
-bool kd_is_sorted_(List x)
+bool kd_is_sorted_(List x, bool parallel)
 {
   switch(arrayvec_dim(x)) {
-  case 1: return kd_is_sorted__<1>(x);
-  case 2: return kd_is_sorted__<2>(x);
-  case 3: return kd_is_sorted__<3>(x);
-  case 4: return kd_is_sorted__<4>(x);
-  case 5: return kd_is_sorted__<5>(x);
-  case 6: return kd_is_sorted__<6>(x);
-  case 7: return kd_is_sorted__<7>(x);
-  case 8: return kd_is_sorted__<8>(x);
-  case 9: return kd_is_sorted__<9>(x);
+  case 1: return kd_is_sorted__<1>(x, parallel);
+  case 2: return kd_is_sorted__<2>(x, parallel);
+  case 3: return kd_is_sorted__<3>(x, parallel);
+  case 4: return kd_is_sorted__<4>(x, parallel);
+  case 5: return kd_is_sorted__<5>(x, parallel);
+  case 6: return kd_is_sorted__<6>(x, parallel);
+  case 7: return kd_is_sorted__<7>(x, parallel);
+  case 8: return kd_is_sorted__<8>(x, parallel);
+  case 9: return kd_is_sorted__<9>(x, parallel);
   default: stop("Invalid dimensions");
   }
 }
@@ -74,7 +79,7 @@ List lex_sort__(List x, bool inplace)
 }
 
 // [[Rcpp::export]]
-List lex_sort_(List x, bool inplace = false)
+List lex_sort_(List x, bool inplace)
 {
   switch(arrayvec_dim(x)) {
   case 1: return lex_sort__<1>(x, inplace);
@@ -208,6 +213,67 @@ IntegerVector kd_rq_indices_(List x, NumericVector lower, NumericVector upper)
   }
 }
 
+template <size_t I>
+List kd_rq_circular__(List x, NumericVector center, double radius)
+{
+  auto p = get_ptr<I>(x);
+  auto q = make_xptr(new arrayvec<I>);
+  auto oi = back_inserter(*q);
+  auto cntr = vec_to_array<I>(center);
+  kd_range_query(begin(*p), end(*p), cntr, radius, oi);
+  return wrap_ptr(q);
+}
+
+// [[Rcpp::export]]
+List kd_rq_circular_(List x, NumericVector center, double radius)
+{
+  switch(arrayvec_dim(x)) {
+  case 1: return kd_rq_circular__<1>(x, center, radius);
+  case 2: return kd_rq_circular__<2>(x, center, radius);
+  case 3: return kd_rq_circular__<3>(x, center, radius);
+  case 4: return kd_rq_circular__<4>(x, center, radius);
+  case 5: return kd_rq_circular__<5>(x, center, radius);
+  case 6: return kd_rq_circular__<6>(x, center, radius);
+  case 7: return kd_rq_circular__<7>(x, center, radius);
+  case 8: return kd_rq_circular__<8>(x, center, radius);
+  case 9: return kd_rq_circular__<9>(x, center, radius);
+  default: stop("Invalid dimensions");
+  }
+}
+
+template <size_t I>
+IntegerVector kd_rqi_circular__(List x, NumericVector center, double radius)
+{
+  auto p = get_ptr<I>(x);
+  auto q = vector<av_iter<I>>();
+  auto oi = back_inserter(q);
+  auto cntr = vec_to_array<I>(center);
+  kd_rq_iters(begin(*p), end(*p), cntr, radius, oi);
+  IntegerVector res(q.size());
+  std::transform(begin(q), end(q), begin(res),
+                 [&](av_iter<I> x){
+                   return distance(p->begin(), x) + 1;
+                 });
+  return res;
+}
+
+// [[Rcpp::export]]
+IntegerVector kd_rqi_circular_(List x, NumericVector center, double radius)
+{
+  switch(arrayvec_dim(x)) {
+  case 1: return kd_rqi_circular__<1>(x, center, radius);
+  case 2: return kd_rqi_circular__<2>(x, center, radius);
+  case 3: return kd_rqi_circular__<3>(x, center, radius);
+  case 4: return kd_rqi_circular__<4>(x, center, radius);
+  case 5: return kd_rqi_circular__<5>(x, center, radius);
+  case 6: return kd_rqi_circular__<6>(x, center, radius);
+  case 7: return kd_rqi_circular__<7>(x, center, radius);
+  case 8: return kd_rqi_circular__<8>(x, center, radius);
+  case 9: return kd_rqi_circular__<9>(x, center, radius);
+  default: stop("Invalid dimensions");
+  }
+}
+
 template <size_t I>
 int kd_nearest_neighbor__(List x, NumericVector v)
 {
@@ -322,36 +388,56 @@ IntegerVector kd_nn_indices_(List x, NumericVector value, int n)
 }
 
 template <size_t I>
-IntegerVector kd_order__(List x, bool parallel)
+IntegerVector kd_order__(List x, bool inplace, bool parallel)
 {
   auto p = get_ptr<I>(x);
+  IntegerVector res(p->size());
+  const vec_type<I>* p0 = &((*p)[0]);
   std::vector<vec_type<I>*> q(p->size());
-  std::transform(begin(*p), end(*p), begin(q),
-                 [](vec_type<I>& x){ return &x; });
+  transform(begin(*p), end(*p), begin(q),
+                 [](vec_type<I>& v){ return &v; });
   if (parallel) kd_sort_threaded(begin(q), end(q));
   else kd_sort(begin(q), end(q));
-  IntegerVector res(q.size());
-  std::transform(begin(q), end(q), begin(res),
-                 [&](vec_type<I>* x){
-                   return distance(&(*p)[0], x) + 1;
-                 });
+  transform(begin(q), end(q), begin(res),
+            [p0](const vec_type<I>* v){
+              return distance(p0, v) + 1;
+            });
+  if (inplace) {
+    auto o = make_xptr(new arrayvec<I>);
+    o->reserve(q.size());
+    auto oi = back_inserter(*o);
+    transform(begin(q), end(q), oi,
+              [](const vec_type<I>* v){
+                return *v;
+              });
+    x["xptr"] = wrap(o);
+    p.release();
+  }
   return res;
 }
 
 // [[Rcpp::export]]
-IntegerVector kd_order_(List x, bool parallel = false)
+IntegerVector kd_order_(List x, bool inplace, bool parallel)
 {
   switch(arrayvec_dim(x)) {
-  case 1: return kd_order__<1>(x, parallel);
-  case 2: return kd_order__<2>(x, parallel);
-  case 3: return kd_order__<3>(x, parallel);
-  case 4: return kd_order__<4>(x, parallel);
-  case 5: return kd_order__<5>(x, parallel);
-  case 6: return kd_order__<6>(x, parallel);
-  case 7: return kd_order__<7>(x, parallel);
-  case 8: return kd_order__<8>(x, parallel);
-  case 9: return kd_order__<9>(x, parallel);
+  case 1: return kd_order__<1>(x, inplace, parallel);
+  case 2: return kd_order__<2>(x, inplace, parallel);
+  case 3: return kd_order__<3>(x, inplace, parallel);
+  case 4: return kd_order__<4>(x, inplace, parallel);
+  case 5: return kd_order__<5>(x, inplace, parallel);
+  case 6: return kd_order__<6>(x, inplace, parallel);
+  case 7: return kd_order__<7>(x, inplace, parallel);
+  case 8: return kd_order__<8>(x, inplace, parallel);
+  case 9: return kd_order__<9>(x, inplace, parallel);
   default: stop("Invalid dimensions");
   }
 }
 
+// [[Rcpp::export]]
+bool using_circular_lexicographical_compare() {
+#ifdef USE_CIRCULAR_LEXICOGRAPHIC_COMPARE
+  return true;
+#else
+  return false;
+#endif
+}
